# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nndcomment.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10nndcomment.proto\"\xd6\x01\n\nNNDComment\x12\x0e\n\x06thread\x18\x01 \x01(\x03\x12\n\n\x02no\x18\x02 \x01(\x05\x12\x0c\n\x04vpos\x18\x03 \x01(\x05\x12\x0c\n\x04\x64\x61te\x18\x04 \x01(\x03\x12\x11\n\tdate_usec\x18\x05 \x01(\x05\x12\x11\n\tanonymity\x18\x06 \x01(\x08\x12\x0f\n\x07user_id\x18\x07 \x01(\t\x12\x0c\n\x04mail\x18\x08 \x01(\t\x12\x0c\n\x04leaf\x18\t \x01(\x05\x12\x0f\n\x07premium\x18\n \x01(\x08\x12\r\n\x05score\x18\x0b \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x0c \x01(\t\x12\x0c\n\x04\x66ork\x18\x0f \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nndcomment_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _NNDCOMMENT._serialized_start=21
  _NNDCOMMENT._serialized_end=235
# @@protoc_insertion_point(module_scope)
